@@ -37,7 +37,8 @@
 #'   serve_table()
 #' @return An edibble design.
 #' @export
-assign_trts <- function(.edibble, order = "random", seed = NULL, constrain = nesting_structure(.edibble), ..., .record = TRUE) {
+assign_trts <- function(.edibble = NULL, order = "random", seed = NULL, constrain = nesting_structure(.edibble), ..., .record = TRUE) {
+  if(is.null(.edibble)) return(structure(match.call(), env = rlang::caller_env(), class = c("edbl_fn", "edbl")))
   not_edibble(.edibble)
   force(constrain) # evaluate this now rather than later
 
@@ -164,7 +165,8 @@ assign_trts <- function(.edibble, order = "random", seed = NULL, constrain = nes
 
 #' @rdname assign_fcts
 #' @export
-assign_units <- function(.edibble, order = "random", seed = NULL, constrain = nesting_structure(.edibble), ..., .record = TRUE) {
+assign_units <- function(.edibble = NULL, order = "random", seed = NULL, constrain = nesting_structure(.edibble), ..., .record = TRUE) {
+  if(is.null(.edibble)) return(structure(match.call(), env = rlang::caller_env(), class = c("edbl_fn", "edbl")))
   not_edibble(.edibble)
   prov <- activate_provenance(.edibble)
   if(.record) prov$record_step()
