@@ -56,7 +56,7 @@ set_fcts <- function(.edibble, ..., .class = NULL,
 #' Constructor for an edibble variable
 #' @importFrom vctrs new_vctr
 #' @noRd
-new_edibble_fct <- function(labels = character(), levels = as.character(unique(labels)),
+new_edibble_fct <- function(labels = character(), levels = sort(as.character(unique(labels))),
                             name = character(), rep = NULL, ..., class = NULL) {
   # don't make the attribute name
   # as this triggers the warning message in ggplot2:
@@ -84,6 +84,9 @@ new_edibble_fct <- function(labels = character(), levels = as.character(unique(l
 #' @export
 as.character.edbl_fct <- function(x, ...) {
   #unname(levels(x)[x])
+  if(inherits(x, "factor")) {
+    return(as.character(format(x)))
+  }
   out <- unclass(x)
   attributes(out) <- NULL
   as.character(out)
@@ -186,6 +189,10 @@ vec_ptype2.integer.edbl_trt <- function(x, y, ...) x
 #' @export
 vec_ptype2.edbl_trt.edbl_trt <- function(x, y, ...) x
 
+#' @export
+vec_ptype2.edbl_rcrd.factor <- function(x, y, ...) y
+#' @export
+vec_ptype2.factor.edbl_rcrd <- function(x, y, ...) x
 #' @export
 vec_ptype2.edbl_rcrd.character <- function(x, y, ...) y
 #' @export
@@ -211,6 +218,10 @@ vec_cast.edbl_trt.integer <- function(x, to, ...) x
 #' @export
 vec_cast.integer.edbl_trt <- function(x, to, ...) as.integer(unclass(x))
 #' @export
+vec_cast.edbl_trt.factor <- function(x, to, ...) x
+#' @export
+vec_cast.factor.edbl_trt <- function(x, to, ...) as.factor(unclass(x))
+#' @export
 vec_cast.edbl_trt.character <- function(x, to, ...) x
 #' @export
 vec_cast.character.edbl_trt <- function(x, to, ...) as.character(unclass(x))
@@ -226,6 +237,10 @@ vec_cast.edbl_unit.integer <- function(x, to, ...) x
 #' @export
 vec_cast.integer.edbl_unit <- function(x, to, ...) as.integer(unclass(x))
 #' @export
+vec_cast.edbl_unit.factor <- function(x, to, ...) x
+#' @export
+vec_cast.factor.edbl_unit <- function(x, to, ...) as.factor(unclass(x))
+#' @export
 vec_cast.edbl_unit.character <- function(x, to, ...) x
 #' @export
 vec_cast.character.edbl_unit <- function(x, to, ...) as.character(unclass(x))
@@ -241,6 +256,10 @@ vec_cast.edbl_rcrd.integer <- function(x, to, ...) x
 #' @export
 vec_cast.integer.edbl_rcrd <- function(x, to, ...) as.integer(unclass(x))
 #' @export
+vec_cast.edbl_rcrd.factor <- function(x, to, ...) x
+#' @export
+vec_cast.factor.edbl_rcrd <- function(x, to, ...) as.factor(unclass(x))
+#' @export
 vec_cast.edbl_rcrd.character <- function(x, to, ...) x
 #' @export
 vec_cast.character.edbl_rcrd <- function(x, to, ...) as.character(unclass(x))
