@@ -114,8 +114,22 @@ with_params <- function(..., .censor = NA, .aggregate = NULL) {
 #' used in the [simulate_process()]. The value should be returned from calling
 #' [with_params()].
 #' @param .seed An optional seed value.
+#' @param .nsim The number of times to simulate data.
+#' @examples
+#' design() %>%
+#'  set_units(unit = 4) %>%
+#'  set_trts(trt = 2) %>%
+#'  allot_table(trt ~ unit) %>%
+#'  set_rcrds(y = unit) %>%
+#'  simulate_process(y = function() {
+#'             res <- rnorm(n())
+#'             res
+#'  }) %>%
+#'  simulate_rcrds(y = with_params(), .nsim = 3)
+#'
+#'
 #' @export
-simulate_rcrds <- function(.data, ..., .seed = NULL) {
+simulate_rcrds <- function(.data, ..., .seed = NULL, .nsim = 1L) {
   dots <- list2(...)
   prov <- activate_provenance(.data)
   prov$save_seed(.seed, type = "simulate_rcrds")
@@ -127,30 +141,35 @@ simulate_rcrds <- function(.data, ..., .seed = NULL) {
                                           .combine_words(srcrds[duplicated(srcrds)], fun = cli::col_blue),
                                           " in multiple processes. The values will be overwritten."))
 
+  data_ret <- .data[0, ]
+  for(isim in seq(.nsim)) {
+    data <- .data
+    for(aprocess in prnames) {
+      process <- prov$get_simulate(aprocess)$process
+      if(is_null(process)) abort(paste0("The supplied process, ", cli::col_blue(aprocess), ", doesn't exist"))
+      srcrds <- prov$get_simulate(aprocess)$rcrds
+      body(process) <- patch_function(process, patch = sprintf("    list2env(setNames(list(mget(ls())), '%s_%d'), envir = result_env)", aprocess, isim),
+                                      position = length(as.list(body(process))) - 1)
 
-  for(aprocess in prnames) {
-    process <- prov$get_simulate(aprocess)$process
-    if(is_null(process)) abort(paste0("The supplied process, ", cli::col_blue(aprocess), ", doesn't exist"))
-    srcrds <- prov$get_simulate(aprocess)$rcrds
-    body(process) <- patch_function(process, patch = sprintf("    list2env(setNames(list(mget(ls())), '%s'), envir = result_env)", aprocess),
-                              position = length(as.list(body(process))) - 1)
+      y <- eval_tidy(do.call(process, dots[[aprocess]]$params), data = data)
 
-    y <- eval_tidy(do.call(process, dots[[aprocess]]$params), data = .data)
-
-    # now assign the values to the data, but apply aggregation then censorship
-    if(grepl("^[.]", aprocess)) {
-      for(acol in colnames(y)) {
-        .data[[acol]] <- get_rcrd_values(acol, prov, dots[[aprocess]]$aggregate,
-                                         .data, y[[acol]], dots[[aprocess]]$censor)
+      # now assign the values to the data, but apply aggregation then censorship
+      if(grepl("^[.]", aprocess)) {
+        for(acol in colnames(y)) {
+          data[[acol]] <- get_rcrd_values(acol, prov, dots[[aprocess]]$aggregate,
+                                          data, y[[acol]], dots[[aprocess]]$censor)
+        }
+      } else {
+        # there should be only one record if it is not a process name
+        arcrd <- srcrds[1]
+        data[[arcrd]] <- get_rcrd_values(arcrd, prov, dots[[aprocess]]$aggregate,
+                                          data, unname(y), dots[[aprocess]]$censor)
       }
-    } else {
-      # there should be only one record if it is not a process name
-      arcrd <- srcrds[1]
-      .data[[arcrd]] <- get_rcrd_values(arcrd, prov, dots[[aprocess]]$aggregate,
-                                        .data, unname(y), dots[[aprocess]]$censor)
     }
+    data$.sim <- isim
+    data_ret <- rbind(data_ret, data)
   }
-  return_edibble_with_graph(.data, prov)
+  return_edibble_with_graph(data_ret, prov)
 }
 
 get_rcrd_values <- function(rname, prov, aggfn, .data, y, censor) {
@@ -215,11 +234,11 @@ get_censored_value <- function(y, valid, censor) {
     } else {
       y[!ind] <- censor
     }
-    y
+    factor(y, levels = valid$values)
   } else if(type=="integer") {
     valid$record <- "numeric"
     res <- get_censored_value(y, valid, censor)
-    round(res)
+    as.integer(round(res))
   } else {
     y
   }
@@ -325,8 +344,9 @@ with_variables <- function(...,
 #' @param ... If supplied, it is a name-value pair where the name should
 #'   correspond to the record factor name and value is the f
 #' @param .seed The seed number.
+#' @param .nsim The number of simulations to run.
 #' @export
-autofill_rcrds <- function(.data, ..., .seed = NULL) {
+autofill_rcrds <- function(.data, ..., .seed = NULL, .nsim = 1L) {
   prov <- activate_provenance(.data)
   prov$save_seed(.seed, type = "autofill_rcrds")
   dots <- list2(...)
@@ -344,7 +364,7 @@ autofill_rcrds <- function(.data, ..., .seed = NULL) {
     default_opts <- with_variables()
     list(type = ifelse(rname %in% names(vrcrds), vrcrds[[rname]]$record, NA),
          rcrd_levels = if(rname %in% names(vrcrds)) vrcrds[[rname]]$values,
-         unit = uname,
+         unit = c(uname, sample(setdiff(unames, uname), size = sample(0:(length(unames) - 1), 1))),
          trts = sample(tnames, size = sample(0:length(tnames), 1)),
          rcrds = character(),
          missing = default_opts$missing,
@@ -399,7 +419,7 @@ autofill_rcrds <- function(.data, ..., .seed = NULL) {
                               "lessThanOrEqual" = vrcrds[[rname]]$value,
                               "lessThan" = vrcrds[[rname]]$value)
         code_list <- effects_code(dep_fcts, .data)
-        code_adjust_y <- sprintf("edibble::rescale_values(y, lower = %f, upper = %f)", valid_lower, valid_upper)
+        code_adjust_y <- sprintf("rescale_values(y, lower = %f, upper = %f)", valid_lower, valid_upper)
       }
     } else if(deps[[rname]]$type=="factor") {
       nlvls <- length(deps[[rname]]$rcrd_levels)
@@ -423,11 +443,10 @@ autofill_rcrds <- function(.data, ..., .seed = NULL) {
 
     process_functions <- c(process_functions, sprintf('
       %s = function() {
-        set.seed(%d)\n%s
+        %s
         %s
         %s
       }', rname,
-        random_seed_number(),
         paste0(code_list$process_code, collapse = "\n"),
         paste0(unlist(code_list$model_code), collapse = "\n        "),
         paste0(code_adjust_y, collapse = "\n")))
@@ -437,7 +456,8 @@ autofill_rcrds <- function(.data, ..., .seed = NULL) {
 
 
   simulate_processes <- sprintf("simulate_process(%s)", paste(process_functions, collapse = ",\n"))
-  simulate_rcrds <- paste0('simulate_rcrds(', paste0(processes, collapse = ',\n                  '), ')')
+  simulate_rcrds <- paste0('simulate_rcrds(', paste0(processes, collapse = ',\n                  '),
+                           ', .nsim = ', .nsim, ')')
   final_code <- parse(text = paste(".data %>%\n  ", simulate_processes, "%>%\n  ", simulate_rcrds))
   eval(final_code)
 }
@@ -449,21 +469,21 @@ effects_code <- function(dep_fcts, .data, nlevels = 1) {
   if(nlevels <= 2) {
     for(fct in dep_fcts) {
       fct_class <- class(.data[[fct]])
-      nfct <- length(unique(.data[[fct]]))
+      nfct <- nlevels(.data[[fct]])
       if("numeric" %in% fct_class) {
         code_list$process_code <- c(code_list$process_code,
                                     sprintf('%s_degree <- sample(1:%d, 1)', fct, ifelse(nfct > 5, 5, nfct - 1)),
                                     paste(sprintf('%s_effects <- as.vector(poly(%s, %s_degree)', fct, fct, fct),
                                           "%*%",
-                                          sprintf('rnorm(%s_degree, 0, 10))', fct)))
+                                          sprintf('rnorm(%s_degree, 0, %.1f))', fct, stats::runif(1, 1, 10))))
         code_list$model_code <- c(code_list$model_code, sprintf("%s_effects", fct))
 
         # logical not accounted for
       } else if(any(c("factor", "character") %in% fct_class)) {
         code_list$process_code <- c(code_list$process_code,
-                                        sprintf('        %s_effects <- setNames(rnorm(%d, 0, 10), unique(%s))',
-                                                fct, nfct, fct))
-        code_list$model_code <- c(code_list$model_code, sprintf("%s_effects[%s]", fct, fct))
+                                        sprintf('        %s_effects <- rnorm(%d, 0, %.1f)',
+                                                fct, nfct, stats::runif(1, 1, 10)))
+        code_list$model_code <- c(code_list$model_code, sprintf("%s_effects[index_levels(%s)]", fct, fct))
       }
     }
     # combine
@@ -475,14 +495,14 @@ effects_code <- function(dep_fcts, .data, nlevels = 1) {
       if("numeric" %in% fct_class) {
         code_list$process_code <- c(code_list$process_code,
                                     sprintf('        %s_degree <- lapply(1:%d, function(i) sample(1:%d, 1))', fct, nlevels, nfct),
-                                    sprintf('        %s_effects <- lapply(1:%d, function(i) as.vector(poly(%s, %s_degree[[i]]) %*% rnorm(%s_degree[[i]], 0, 10)))', fct, nlevels, fct, fct, fct))
+                                    sprintf('        %s_effects <- lapply(1:%d, function(i) as.vector(poly(%s, %s_degree[[i]]) %*% rnorm(%s_degree[[i]], 0, %.1f)))', fct, nlevels, fct, fct, fct, stats::runif(1, 1, 10)))
 
         code_list$model_code <- c(code_list$model_code, sprintf("%s_effects[[i]]", fct))
       } else if(any(c("factor", "character") %in% fct_class)) {
         code_list$process_code <- c(code_list$process_code,
-                                    sprintf('        %s_effects <- lapply(1:%d, function(i) setNames(rnorm(%d, 0, 10), unique(%s)))',
-                                            fct, nlevels, nfct, fct))
-        code_list$model_code <- c(code_list$model_code, sprintf("%s_effects[[i]][%s]", fct, fct))
+                                    sprintf('        %s_effects <- lapply(1:%d, function(i) rnorm(%d, 0, %.1f))',
+                                            fct, nlevels, nfct, stats::runif(1, 1, 10)))
+        code_list$model_code <- c(code_list$model_code, sprintf("%s_effects[[i]][index_levels(%s)]", fct, fct))
       }
     }
     # combine
@@ -497,6 +517,7 @@ effects_code <- function(dep_fcts, .data, nlevels = 1) {
 #' @param data An edibble data frame.
 #' @param process The process name. Typically the name of the process. If unknown,
 #'  leave this empty.
+#' @param sim The simulation number. Default is 1.
 #' @export
 examine_process <- function(data, process = NULL) {
   prov <- activate_provenance(data)
@@ -514,15 +535,18 @@ examine_process <- function(data, process = NULL) {
 
 #' @rdname examine_process
 #' @export
-examine_process_values <- function(data, process = NULL) {
+examine_process_values <- function(data, process = NULL, sim = 1L) {
   prov <- activate_provenance(data)
-  res <- prov$get_simulate_result_env(process)
+  if(is.null(process)) pname <- NULL else pname <- paste0(process, "_", sim)
+  res <- prov$get_simulate_result_env(pname)
   if(is_null(res)) {
     warning("There is no simulation process stored.")
     NULL
   } else if(is.environment(res)) {
+    pnames <- ls(envir = res, all.names = TRUE)
+    pnames <- unique(gsub("_[0-9]+$", "", pnames))
     abort(paste0("You need to specify a process name. The available process names are: ",
-                .combine_words(ls(envir = res, all.names = TRUE), fun = cli::col_blue), "."))
+                .combine_words(pnames, fun = cli::col_blue), "."))
   } else {
     res
   }
